// $Id: ServerCommandModel.cpp,v 1.25 2014/02/10 10:34:30 hupereir Exp $

/******************************************************************************
*
* Copyright (C) 2002 Hugo PEREIRA <mailto: hugo.pereira@free.fr>
*
* This is free software; you can redistribute it and/or modify it under the
* terms of the GNU General Public License as published by the Free Software
* Foundation; either version 2 of the License, or (at your option) any later
* version.
*
* This software is distributed in the hope that it will be useful, but WITHOUT
* Any WARRANTY; without even the implied warranty of MERCHANTABILITY or
* FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
* for more details.
*
* You should have received a copy of the GNU General Public License along with
* this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include "ServerCommandModel.h"
#include "ServerCommandModel.moc"

#include "Color.h"
#include "CustomPixmap.h"
#include "Singleton.h"
#include "TimeStamp.h"
#include "X11Interface.h"
#include "XmlOptions.h"

#include <QApplication>

//____________________________________________________
const QString ServerCommandModel::columnTitles_[ ServerCommandModel::nColumns ] =
{
    tr( "Name" ),
    tr( "User" ),
    tr( "Display" ),
    tr( "Command" ),
    tr( "Process Id" ),
    tr( "Time" )
};

//_______________________________________________________________
ServerCommandModel::ServerCommandModel( QObject* parent ):
    TreeModel<Server::ServerCommand>( parent ),
    Counter( "ServerCommandModel" )
{
    Debug::Throw( "ServerCommandModel::ServerCommandModel.\n" );
    connect( Singleton::get().application(), SIGNAL(configurationChanged()), SLOT(_updateConfiguration()) );
    _updateConfiguration();
}

//__________________________________________________________________
QVariant ServerCommandModel::data( const QModelIndex& index, int role ) const
{

    Debug::Throw(2) << "ServerCommandModel::data - row: " << index.row() << " column: " << index.column() << " role: " << role << endl;

    // check index, role and column
    if( !index.isValid() ) return QVariant();

    // return text associated to file and column
    switch( role )
    {

        case Qt::DisplayRole:
        {

            // retrieve associated file info
            const Server::ServerCommand& command( _find( index.internalId() ).get() );
            switch( index.column() )
            {

                case Time: return command.timeStamp().toString();
                case Name: return command.id().name();
                case User: return command.id().userName();
                case Display: return command.id().display();
                case Command: return command.commandName();
                case Pid: return command.id().processId();
                default: break;
            }

            break;

        }

        case Qt::ForegroundRole:
        {

            const Server::ServerCommand& command( _find( index.internalId() ).get() );
            if( command.command() == Server::ServerCommand::Identify && identifyColor_.isValid() ) return identifyColor_;
            else if( command.command() == Server::ServerCommand::IdentifyServer && serverColor_.isValid() ) return serverColor_;
            else return QVariant();

        }

        case Qt::DecorationRole:
        {
            if( showIcons_ && index.column() == Name )
            {
                const Server::ServerCommand& command( _find( index.internalId() ).get() );
                return _icon( command.id().processId() );
            }
            break;
        }

        default: break;

    }

    return QVariant();

}

//__________________________________________________________________
QVariant ServerCommandModel::headerData(int section, Qt::Orientation orientation, int role) const
{

    if(
        orientation == Qt::Horizontal &&
        role == Qt::DisplayRole &&
        section >= 0 &&
        section < int(nColumns) )
    { return columnTitles_[section]; }

    // return empty
    return QVariant();

}


//__________________________________________________________________
void ServerCommandModel::remove( ServerCommandModel::List values )
{

    // remove icons
    foreach( const Server::ServerCommand& command, values )
    { icons_.remove( command.id().processId() ); }

    TreeModel<Server::ServerCommand>::remove( values );

}

//__________________________________________________________________
void ServerCommandModel::clear( void )
{
    TreeModel<Server::ServerCommand>::clear();
    icons_.clear();
}

//__________________________________________________________________
QIcon ServerCommandModel::_icon( qint64 pid ) const
{

    const IconMap::const_iterator iter = icons_.find( pid );
    if( iter != icons_.end() ) return iter.value();

    QIcon icon( X11Interface().findIcon( pid ) );
    if( icon.isNull() ) return emptyIcon_;
    else {

        const_cast<IconMap*>( &icons_ )->insert( pid, icon );
        return icon;

    }

}

//__________________________________________________________
void ServerCommandModel::_updateConfiguration(void)
{

    Debug::Throw( "JobModel::_updateConfiguration.\n" );
    showIcons_ = XmlOptions::get().get<bool>( "SHOW_ICONS" );

    // empty icon
    const int iconSize( XmlOptions::get().get<unsigned int>( "LIST_ICON_SIZE" ) );
    iconSize_ = QSize( iconSize, iconSize );

    // create empty pixmap
    emptyIcon_ = CustomPixmap().empty( iconSize_ );

    // colors
    serverColor_ = XmlOptions::get().get<Base::Color>( "SERVER_COLOR" );
    identifyColor_ = XmlOptions::get().get<Base::Color>( "IDENTIFY_COLOR" );

}

//________________________________________________________
bool ServerCommandModel::SortFTor::operator () ( Server::ServerCommand first, Server::ServerCommand second ) const
{

    if( order_ == Qt::DescendingOrder ) std::swap( first, second );
    switch( type_ )
    {

        case Name: return first.id().name() < second.id().name();
        case User: return first.id().user() < second.id().user();
        case Display: return first.id().display() < second.id().display();
        case Command: return first.command() == Server::ServerCommand::IdentifyServer || first.command() < second.command();
        case Time: return first.timeStamp() < second.timeStamp();
        case Pid: return first.id().processId() < second.id().processId();
        default: return false;

    }

}
